#ifndef AFFICHABLE_H
#define AFFICHABLE_H

#include <SDL/SDL.h>
#include <GL/glew.h>
#include "matr9f.h"
#include "octree.h"

// regroupe les indices des sommets du triangle
class Triangle{
    public:
    Triangle(){}
    inline Triangle(int a0, int b0, int c0){a=a0;b=b0;c=c0;}
    int a,b,c;

    inline void print(){printf("Triangle(%d,%d,%d)\n",a,b,c);}
};

// structure comprenant le sommet avec sa couleur, et la normale au sommet
class Vertexcolornormal : public Vect3f{
    public:
    int color;
    Vect3f normale;

    inline unsigned char& red(){return *((unsigned char*)&color);}
    inline unsigned char& green(){return *((unsigned char*)&color+1);}
    inline unsigned char& blue(){return *((unsigned char*)&color+2);}
    inline unsigned char& alpha(){return *((unsigned char*)&color+3);}

    inline Vertexcolornormal():Vect3f(),normale(){color=0xff808080;}
    inline Vertexcolornormal(Vect3f u, int col, Vect3f n):Vect3f(u),normale(n){color=col;}
    inline Vertexcolornormal(Vect3f u, int col):Vect3f(u){color=col;}
    inline Vertexcolornormal(Vect3f u):Vect3f(u){color=0xff808080;}
};

// pour aficher des points
class NuageDePoints{
        private:
    GLuint buffer;
    int nbrvertex;
        public:
    NuageDePoints(Vect3f* tab,int nbrvecteurs,int nbrmaxvecteurs);
    NuageDePoints(Vertexcolornormal* tab,int nbrvecteurs,int nbrmaxvecteurs);

    // Pour afficher les points d'un Octree
    template<class T>    NuageDePoints(Traceur<T> &trac){
        Vect3f a;
        int indice=(*trac.gethead()).compteextremite();

        Vertexcolornormal *vertextab=new Vertexcolornormal[indice*4];
        indice = 0;
        trac.firstlastcube();do{
            a=trac.getcoordonne();
            float l=trac.getlargeur();
            vertextab[indice*4] = Vertexcolornormal(a,0xff00ff00);
            vertextab[indice*4+1] = Vertexcolornormal(a+Vect3f(l,0,0),0xff00ff00);
            vertextab[indice*4+2] = Vertexcolornormal(a+Vect3f(0,l,0),0xff00ff00);
            vertextab[indice*4+3] = Vertexcolornormal(a+Vect3f(0,0,l),0xff00ff00);
            indice++;
        }while(trac.nextlastcube());

        nbrvertex=indice*4;
        //génération du buffers d'informations de vertex
        glGenBuffers(1,&buffer);
        glBindBuffer(GL_ARRAY_BUFFER, buffer);
        glBufferData(GL_ARRAY_BUFFER, nbrvertex*sizeof(Vertexcolornormal), NULL, GL_DYNAMIC_DRAW);
        glBufferSubData(GL_ARRAY_BUFFER, 0, nbrvertex*sizeof(Vertexcolornormal), (GLfloat*)vertextab);
        delete[] vertextab;
    }

    void affiche()const;

    void subdata(Vertexcolornormal* tab,int nbrvecteurs);

    void realloc(Vertexcolornormal* tab,int nbrvecteurs,int nbrmaxvecteurs);

    inline ~NuageDePoints(){glDeleteBuffers(1,&buffer);}
};

// pour afficher un maillage
class Toile{
    GLuint buffer[2];
    public:

    int nbrvertex;
    GLfloat* vertextab;

    int nbrsegments;
    int *segmenttab;
        public:
    Toile(Vertexcolornormal* tab,int nbrvecteurs,int* indices,int nbrlignes);


    void affiche()const;
    void grow(float dl);


    inline ~Toile(){
        glDeleteBuffers(2,buffer);
        delete[] (Vertexcolornormal*)vertextab;
        delete[] segmenttab;}
};

#endif
