#include <SDL/SDL.h>
#include <GL/glew.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>

#include "octree.h"
#include "potentiel.h"
#include "affichable.h"
#include "contour_dual.h"


#define LARGEUR_FENETRE 900
#define HAUTEUR_FENETRE 720
#define DELAY_FRAME 30


/** biblihotèques linux :
	SDLmain
	SDL
	GLU
	GL
	GLEW
**/
/** biblihotèques windows avec compilation directe de glew
    mingw32
    SDLmain
    SDL
    opengl32
    glu32
**/


int boucle(SDL_Event* event);

int main(int argc, char *argv[])
{
    int k;
    SDL_Event event;



    SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);// initialisation de la SDL
    SDL_WM_SetCaption("graphisme", NULL);// création d'une fenêtre

    SDL_SetVideoMode(LARGEUR_FENETRE, HAUTEUR_FENETRE, 32, SDL_OPENGL);

    glewInit();// initialisation de GLEW

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    gluPerspective(70,(double)LARGEUR_FENETRE/HAUTEUR_FENETRE,1,1000);



    glEnable(GL_DEPTH_TEST);// Z buffer
    glEnable(GL_LIGHTING);// éclairage
    glEnable(GL_LIGHT0);// lumière suivant z
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);



    int t0 = SDL_GetTicks();

    Verre v;
    Contour_dual cont1(&v,0.,3,7,6.);// cont1 : le verre
    /*triangule un verre dans un espace de largeur 6.
    Avec comme profondeur de détection 3,
    et comme profondeur de voxel 7
    avec comme équation v=0.
    */
    t0=SDL_GetTicks()-t0;
    printf("temps de construction du contour : %d ms\n",t0);
    cont1.printstats();// affichage des caractéristiques de la triangulation


    Toile toile1(cont1.gettoile(0xff000000));// On récupère la toile du maillage
    toile1.grow(0.005);// on fait ressortir la toile pour ne pas qu'elle soit confondue avec l'image


    k=0;

    Uint32 temps = SDL_GetTicks()-50;
    Uint32 tempspasse;
    int a1=0,a2=0;

    while (boucle(&event))
    {
        glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

        glMatrixMode( GL_MODELVIEW );
        glLoadIdentity();

        gluLookAt(0,0,3,0,0,0,0,1,0);//positionne la caméra, ici à une distance de "3" de l'origine
        glPushMatrix();

        if(event.button.x!=0 && event.button.y!=0){a1=event.button.x;a2=event.button.y;}
        glRotated(a1*0.5,0,1,0);// pivote la figure en fonction du placement de la sourie
        glRotated(a2*0.5,1,0,0);


        Potentiel potblob2(6.);// création d'un potentiel de blobs
        potblob2.addblob(Blob(Vect3f(.5*cos(0.02*k),sin(0.02*k),0),Matr9f(1,1,1)));
        potblob2.addblob(Blob(-Vect3f(.5*cos(0.02*k),sin(0.02*k),0),Matr9f(1,1,1)));
        potblob2.addblob(Blob(Vect3f(cos(0.015*k),0,sin(0.015*k)),Matr9f(1,1,1)));

        Contour_dual cont2(&potblob2,0.5,3,6,6.);

        // ajout des couleurs
        Couleurs c(potblob2.trac);//création d'un champ de couleur
        c.addcouleur(Blob(Vect3f(.5*cos(0.02*k),sin(0.02*k),0),Matr9f(1,1,1)),0xff00ff00);
        c.addcouleur(Blob(-Vect3f(.5*cos(0.02*k),sin(0.02*k),0),Matr9f(1,1,1)),0xffff0000);
        c.addcouleur(Blob(Vect3f(cos(0.015*k),0,sin(0.015*k)),Matr9f(1,1,1)),0xff0000ff);
        cont2.setcolor(c);// calcul des couleurs pour chaque sommet à partir du champ


        Toile toile2(cont2.gettoile(0x80000000));// On récupère la toile du maillage
        toile2.grow(0.005);// on fait ressortir la toile pour ne pas qu'elle soit confondue avec l'image

        cont2.affiche();//affichage des blobs mouvants
        toile2.affiche();//affichage de la toile


        //cont1.affiche();//affichage du Verre
        //toile1.affiche();//affichage de la toile



        glPopMatrix();

        tempspasse = temps - SDL_GetTicks() + DELAY_FRAME;
        if (tempspasse < 10000)SDL_Delay(tempspasse);// si jamais tempspasse est négatif on n'attends pas
        temps = SDL_GetTicks();

        k++;


        glFlush();// mise à jour de l'écran
        SDL_GL_SwapBuffers();
    }


    return 0;
}

int boucle(SDL_Event* event){
    int continuer=1;
    while (SDL_PollEvent(event)){
    switch ((*event).type)
        {
        case SDL_QUIT:
            continuer = 0;break;
        case SDL_KEYDOWN: /* Si appui d'une touche */
            switch ((*event).key.keysym.sym)
                {
                case SDLK_ESCAPE: /* Appui sur la touche Echap, on arrête le programme */
                    continuer = 0;break;
                default: ;
                }
            break;
        default: ;
        }
    }
    return continuer;
}
